/*
 * Copyright (C) 2017 Centro de Computacao Cientifica e Software Livre
 * Departamento de Informatica - Universidade Federal do Parana
 *
 * This file is part of blendb.
 *
 * blendb is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * blendb is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with blendb.  If not, see <http://www.gnu.org/licenses/>.
 */

import { Middleware } from "../types";
import { Adapter } from "../../core/adapter";
import { PostgresAdapter } from "../../adapter/postgres";
import { MonetAdapter, MonetConfig } from "../../adapter/monet";
import { ElasticsearchAdapter } from "../../adapter/elasticsearch";
import { PoolConfig } from "pg";
import { Connection } from "../../util/configParser";
import { ConfigOptions } from "elasticsearch";

/**
 * Creates a PostgreSQL adapter and middleware that
 * inserts the adapter into the request objects.
 * @param config - Parameters required to connect in database.
 */
export function PostgresMw(config: Connection): Middleware {
    let parsedConfig: PoolConfig = {
        user: config.user,
        database: config.database,
        password: config.password,
        host: config.host,
        port: config.port,
        max: 10,
        idleTimeoutMillis: 3000
    };
    let adapter: Adapter = new PostgresAdapter(parsedConfig);
    return function postgresMiddleware(req, res, next) {
        req.adapter = adapter;
        next();
    };

}

/**
 * Creates a MonetDB adapter and middleware that
 * inserts the adapter into the request objects.
 * @param config - Parameters required to connect in database.
 */
export function MonetMw(config: Connection): Middleware {
    let parsedConfig: MonetConfig = {
        user: config.user,
        dbname: config.database,
        password: config.password,
        host: config.host,
        port: config.port,
    };
    let adapter: Adapter = new MonetAdapter(parsedConfig);
    return function monetMiddleware(req, res, next) {
        req.adapter = adapter;
        next();
    };

}

/*Elastic */

export function ElasticsearchMw(config: Connection): Middleware {
	let parsedConfig: ConfigOptions = {
		host: config.host + ":" + config.port,
		maxRetries: 5,
  		requestTimeout: 60000,
  		sniffOnStart: true,
	};
	let adapter: Adapter = new ElasticsearchAdapter(parsedConfig);
	return function elasticsearchMiddleware(req, res, next) {
		req.adapter = adapter;
		next();
	};

}
